@@ -1,7 +1,7 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: debug.h,v 1.4 1999/01/18 19:43:40 kahn Exp kahn $
+ * $Id: debug.h,v 1.8 1999/09/12 03:44:55 kahn Exp kahn $
  *
- * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
+ * Copyright (C) 1998 Andy C. Kahn
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -44,6 +44,9 @@ extern long dbg_flags;
 #define DBG_TOOLBAR	0x00000800
 #define DBG_WIN		0x00001000
 #define DBG_HTML	0x00002000
+#define DBG_HTMLVIEW	0x00004000
+#define DBG_UNDO	0x00008000
+#define DBG_SRCCTRL	0x00010000
 
 #define DBG_PRINTF(options, s)	\
 	if (dbg_flags & (options))	\
@@ -64,6 +67,9 @@ extern long dbg_flags;
 #define GNPDBG_TOOLBAR(s)	DBG_PRINTF(DBG_TOOLBAR, s);
 #define GNPDBG_WIN(s)		DBG_PRINTF(DBG_WIN,     s);
 #define GNPDBG_HTML(s)		DBG_PRINTF(DBG_HTML,    s);
+#define GNPDBG_HTMLVIEW(s)	DBG_PRINTF(DBG_HTMLVIEW,s);
+#define GNPDBG_UNDO(s)		DBG_PRINTF(DBG_UNDO,	s);
+#define GNPDBG_SRCCTRL(s)	DBG_PRINTF(DBG_SRCCTRL,	s);
 
 #else	/* NO DEBUG */
 
@@ -84,6 +90,9 @@ extern long dbg_flags;
 #define GNPDBG_TOOLBAR(s)
 #define GNPDBG_WIN(s)
 #define GNPDBG_HTML(s)
+#define GNPDBG_HTMLVIEW(s)
+#define GNPDBG_UNDO(s)
+#define GNPDBG_SRCCTRL(s)
 
 #endif	/* APP_DEBUG */
 
