@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: doc.c,v 1.87 2000/03/01 01:46:03 kahn Exp kahn $
+ * $Id: doc.c,v 1.96 2000/04/25 05:11:09 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn
  *
@@ -343,8 +343,7 @@ doc_close_all_cb(GtkWidget *wgt, gpointer cbdata)
 		return;
 
 	msgbar_printf(w, _("Closing all files... (please wait)"));
-	while (gtk_events_pending())
-		gtk_main_iteration_do(TRUE);
+	gtk_events_flush();
 	allclosed = doc_close_all_common(w);
 
 	/*
@@ -405,7 +404,7 @@ doc_close_all_common(win_t *w)
 
 		while (w->numdoc > 0) {
 			allclosed = doc_close_common(w, FALSE);
-			gtk_main_iteration_do(FALSE);
+			gtk_events_flush();
 			if (allclosed == FALSE)	/* not closed */
 				break;
 		}
@@ -581,9 +580,11 @@ doc_info_cb(GtkWidget *wgt, gpointer cbdata)
 	doc_info_table_add(table, di, _("Last modified:"), 0, 1, 15, 16, TRUE);
 	doc_info_table_add(table, di, _("Last status change:"),0, 1, 16, 17, TRUE);
 #ifndef __EMX__
-		/* OS/2's EMX (Unix porting libs) does not have these */
-	doc_info_table_add(table, di, _("File block size:"),   0, 1, 17, 18, TRUE);
-	doc_info_table_add(table, di, _("Blocks allocated:"),  0, 1, 18, 19, TRUE);
+	/* OS/2's EMX (Unix porting libs) does not have these */
+	doc_info_table_add(table, di, _("File block size:"),
+			   0, 1, 17, 18, TRUE);
+	doc_info_table_add(table, di, _("Blocks allocated:"),
+			   0, 1, 18, 19, TRUE);
 #endif
 
 	s = file_full_pathname_make(d->fname);
@@ -823,11 +824,16 @@ doc_list_remove(win_t *w, int num)
 void
 doc_list_show(GtkWidget *wgt, gpointer cbdata)
 {
+#ifdef USE_LIBGLADE
+	GladeXML *xml;
+#else
+	GtkWidget *scrolled_win;
+	char *titles[] = { N_(" # "), N_(" Size (bytes) "), N_(" File name ") };
+#endif
 	int num;
 	GSList *dp;
 	doc_t *d;
-	GtkWidget *tmp, *vbox, *scrolled_win;
-	char *titles[] = { N_(" # "), N_(" Size (bytes) "), N_(" File name ") };
+	GtkWidget *tmp, *vbox;
 	win_t *w = (win_t *)cbdata;
 
 	g_assert(w != NULL);
@@ -835,15 +841,20 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	if (misc_show_and_raise(w->dlw))
 		return;
 
+	msgbar_printf(w, _("Creating doc list... (this might take a minute)"));
+
+#ifdef USE_LIBGLADE
+	xml = my_glade_xml_get("Doc List");
+	w->dlw = my_glade_widget_get(xml, "Doc List", 0);
+	w->dlw_data = my_glade_widget_get(xml, "doclistclist", 0);
+	vbox = my_glade_widget_get(xml, "doclistvbox", 0);
+	gtk_object_destroy(GTK_OBJECT(xml));
+#else
 	for (num = 0; num < sizeof(titles)/sizeof(char*); num++)
 		titles[num] = gettext(titles[num]);
 
-	msgbar_printf(w, _("Creating doc list... (this might take a minute)"));
-
 	w->dlw = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(w->dlw), _("Currently Open Files"));
-	(void)gtk_signal_connect(GTK_OBJECT(w->dlw), "destroy",
-				 GTK_SIGNAL_FUNC(doc_list_destroy), cbdata);
 
 	vbox = gtk_vbox_new(FALSE, 10);
 	gtk_container_add(GTK_CONTAINER(w->dlw), vbox);
@@ -852,19 +863,20 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 
 	w->dlw_data = gtk_clist_new_with_titles(FLW_NUM_COLUMNS, titles);
 	gtk_widget_show(w->dlw_data);
+	scrolled_win = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_container_add(GTK_CONTAINER(scrolled_win), w->dlw_data);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0);
+	gtk_widget_show(scrolled_win);
+	tmp = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, TRUE, 0);
+	gtk_widget_show(tmp);
+#endif
 
-#if 0
-	GtkStyle *style;
-	d = DOC_CURRENT(w);
-	style = gtk_style_copy(GTK_WIDGET(d->data)->style);
-	gdk_font_unref(style->font);
-	gtk_widget_push_style(style);
-#ifdef GTK_HAVE_FEATURES_1_1_0
-	/* XXX - there seems to be a problem in gtk-1.0.x */
-	gtk_widget_set_style(w->dlw_data, style);
-#endif
-	gtk_widget_pop_style();
-#endif
+	(void)gtk_signal_connect(GTK_OBJECT(w->dlw), "destroy",
+				 GTK_SIGNAL_FUNC(doc_list_destroy), cbdata);
 
 	tmp = w->dlw_data;
 	gtk_clist_column_titles_passive(GTK_CLIST(tmp));
@@ -877,19 +889,12 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	gtk_clist_set_column_width(GTK_CLIST(tmp), FnumCol, 25);
 	gtk_clist_set_column_width(GTK_CLIST(tmp), FsizeCol, 90);
 #endif
-	scrolled_win = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_win),
-				       GTK_POLICY_AUTOMATIC,
-				       GTK_POLICY_AUTOMATIC);
-	gtk_container_add(GTK_CONTAINER(scrolled_win), tmp);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled_win, TRUE, TRUE, 0);
-	gtk_widget_show(scrolled_win);
 
 	gtk_clist_freeze(GTK_CLIST(w->dlw_data));
 	for (num = 0, dp = w->doclist; dp; num++, dp = dp->next) {
 		d = (doc_t *)(dp->data);
 		doc_list_add(w, d, num, doc_basefname(d), APPEND, FALSE);
-		gtk_main_iteration_do(FALSE);
+		gtk_events_flush();
 	}
 	gtk_clist_thaw(GTK_CLIST(w->dlw_data));
 
@@ -900,17 +905,13 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 	(void)gtk_signal_connect(GTK_OBJECT(w->dlw_data), "select_row",
 				 GTK_SIGNAL_FUNC(doc_list_select), w);
 
-	tmp = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, TRUE, 0);
-	gtk_widget_show(tmp);
-
 	toolbar_create(w, vbox, dlw_tbdata, NULL);
 
 	/*
 	 * there should be a way to auto-detect how wide to make these widgets
 	 * instead of having to use arbitrary values
 	 */
-	gtk_widget_set_usize(w->dlw, 250, 350);
+	gtk_window_set_default_size(GTK_WINDOW(w->dlw), 250, 350);
 
 	/*
 	 * since we're currently inserting documents at the head of the
@@ -925,6 +926,47 @@ doc_list_show(GtkWidget *wgt, gpointer cbdata)
 #endif	/* USE_DOCLIST */
 
 
+static gboolean
+doc_text_scroll(doc_t *d, bool_t up)
+{
+	float val;
+	GtkAdjustment *adj;
+
+	if (!GTK_IS_TEXT(d->data))
+		return FALSE;
+
+	adj = GTK_ADJUSTMENT(GTK_TEXT(d->data)->vadj);
+	val = (up) ? adj->value - (adj->page_increment / 2) :
+		adj->value + (adj->page_increment / 2);
+	gtk_adjustment_set_value(adj, val);
+
+	return TRUE;
+} /* doc_text_scroll */
+
+
+static gboolean
+doc_text_event_cb(GtkWidget *wgt, GdkEvent *ev, gpointer cbdata)
+{
+	win_t *w = (win_t *)cbdata;
+
+	if (ev->type == GDK_BUTTON_PRESS) {
+		GdkEventButton *event = (GdkEventButton *)ev;
+		switch (event->button) {
+		case 3:
+			quickmenu_show(w->quickmenu, ev);
+			return TRUE;
+		case 4:
+			return doc_text_scroll(DOC_CURRENT(w), TRUE);
+		case 5:
+			return doc_text_scroll(DOC_CURRENT(w), FALSE);
+		default:
+			break;
+		}
+	}
+	return FALSE;
+} /* doc_text_event_cb */
+
+
 /*
  * PUBLIC: doc_new
  *
@@ -968,6 +1010,8 @@ doc_new(win_t *w, char *fname, doc_type_t doctype, long flags)
 		gtk_text_set_word_wrap(GTK_TEXT(d->data), IS_USE_WORDWRAP());
 		gtk_widget_show(d->data);
 #if 0
+		gtk_signal_connect_after(GTK_OBJECT(d->data), "key_press_event",
+					 GTK_SIGNAL_FUNC(gtk_false), NULL);
 		gtk_signal_connect(GTK_OBJECT(d->data), "key_press_event",
 				   GTK_SIGNAL_FUNC(doc_key_press), NULL);
 #endif
@@ -981,9 +1025,8 @@ doc_new(win_t *w, char *fname, doc_type_t doctype, long flags)
 #endif	/* USE_UNDOREDO */
 
 		/* quickmenu */
-		(void)gtk_signal_connect_object(GTK_OBJECT(d->data), "event",
-						GTK_SIGNAL_FUNC(quickmenu_show),
-						GTK_OBJECT(w->quickmenu));
+		(void)gtk_signal_connect(GTK_OBJECT(d->data), "event",
+					 GTK_SIGNAL_FUNC(doc_text_event_cb), w);
 		break;
 #ifdef USE_GTKXMHTML
 	case DocXmHtml:
@@ -1183,8 +1226,12 @@ doc_drag_data_get(GtkWidget *wgt, GdkDragContext *context,
 	 */
 	if (d->fname) {
 		full = file_full_pathname_make(d->fname);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+		uri = g_strdup_printf("file:%s\r\n", full);
+#else
 		uri = g_new(char, 5 + strlen(full) + 3);
 		sprintf(uri, "file:%s\r\n", full);
+#endif
 		g_free(full);
 		gtk_selection_data_set(selection_data, selection_data->target,
 				       8, (guchar *)uri, strlen(uri));
@@ -1405,7 +1452,7 @@ doc_saveas_common(win_t *w)
 	if (misc_show_and_raise(w->saveas)) {
 		/* allow only one saveas box.  wait until that one goes away */
 		while (w->saveas)
-			gtk_main_iteration_do(TRUE);
+			gtk_events_flush();
 	}
 
 	d = DOC_CURRENT(w);
@@ -1426,7 +1473,7 @@ doc_saveas_common(win_t *w)
 	gtk_widget_show(w->saveas);
 
 	while (w->saveas != NULL)
-		gtk_main_iteration_do(TRUE);
+		gtk_events_flush();
 
 	return (d->changed_id != last_id);
 } /* doc_saveas_common */
@@ -1718,10 +1765,14 @@ doc_close_verify(win_t *w)
 	int ret;
 	char *title, *msg, *fn;
 	doc_t *d = DOC_CURRENT(w);
-    char *close_title = _("Save File");
-    char *close_msg = _("has been modified. \n Do you wish to save it?");
+	char *close_title = _("Save File");
+	char *close_msg = _("has been modified. \n Do you wish to save it?");
 
 	fn = d->fname;
+#ifdef GTK_HAVE_FEATURES_1_1_0
+	title = g_strdup_printf("%s '%s'?", close_title, fn ? fn : UNTITLED);
+	msg   = g_strdup_printf(" '%s' %s ", fn ? fn : UNTITLED, close_msg);
+#else
 	if (fn) {
 		title = g_new(char, strlen(close_title) + strlen(fn) + 5);
 		msg   = g_new(char, strlen(close_msg) + strlen(fn) + 6);
@@ -1731,6 +1782,7 @@ doc_close_verify(win_t *w)
 	}
 	sprintf(title, "%s '%s'?", close_title, fn ? fn : UNTITLED);
 	sprintf(msg  , " '%s' %s ", fn ? fn : UNTITLED, close_msg);
+#endif
 
 	ret = do_dialog_yes_no_cancel(title, msg);
 
@@ -2183,8 +2235,8 @@ doc_saveas_ok(GtkWidget *wgt, gpointer cbdata)
 	doc_t *d;
 	bool_t saved;
 	char *fname, *title, *msg;
-    char *ow_title = _("Overwrite");
-    char *ow_msg = _("exists.  Overwrite?");
+	char *ow_title = _("Overwrite");
+	char *ow_msg = _("exists.  Overwrite?");
 
 	win_t *w = (win_t *)cbdata;
 
@@ -2196,10 +2248,15 @@ doc_saveas_ok(GtkWidget *wgt, gpointer cbdata)
 		return;
 
 	if (file_exist(fname)) {
+#ifdef GTK_HAVE_FEATURES_1_1_0
+		title = g_strdup_printf("%s '%s'?", ow_title, fname);
+		msg = g_strdup_printf(" '%s' %s ", fname, ow_msg);
+#else
 		title = g_new(char, strlen(ow_title) + strlen(fname) + 5);
 		msg   = g_new(char, strlen(ow_msg) + strlen(fname) + 6);
 		sprintf(title, "%s '%s'?", ow_title, fname);
 		sprintf(msg  , " '%s' %s ", fname, ow_msg);
+#endif
 
 		ret = do_dialog_yes_no_cancel(title, msg);
 
