@@ -1,5 +1,5 @@
 /* vi:set ts=8 sts=0 sw=8:
- * $Id: win.c,v 1.67 2000/02/28 17:24:19 kahn Exp kahn $
+ * $Id: win.c,v 1.78 2000/04/21 17:01:26 kahn Exp kahn $
  *
  * Copyright (C) 1998 Andy C. Kahn
  *
@@ -17,10 +17,15 @@
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
+#include <unistd.h>
 #include "main.h"
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
 #ifdef USE_GNOME
 #include <gnome.h>
 #endif
@@ -163,7 +168,7 @@ toolbar_data_t main_tbdata[] = {
 		N_("Find Text"),
 		MAIN_TB_STR_FIND,
 		"tb_find.xpm",
-		GTK_SIGNAL_FUNC(search_cb),
+		GTK_SIGNAL_FUNC(search_search_cb),
 		NULL,
 		MAIN_TB_STR_FIND
 	},
@@ -281,7 +286,7 @@ static guint autosave_id = 0;		/* doc autosave timeout id */
 /*** local function prototypes ***/
 static void	notebook_init(win_t *w);
 static int	win_delete_event_cb(GtkWidget *wgt, GdkEvent *, gpointer);
-static bool_t	win_close_common(win_t *w);
+static bool_t	win_close_common(win_t *w, bool_t exiting);
 #ifdef USE_WINLIST
 static void	win_list_destroy(GtkWidget *wgt, gpointer cbdata);
 static void	win_list_remove(win_t *w);
@@ -310,6 +315,11 @@ static GList *	gnome_uri_list_extract_filenames(const gchar* uri_list);
 static GList *	gnome_uri_list_extract_uris(const gchar* uri_list);
 static void	gnome_uri_list_free_strings(GList *list);
 #endif
+#ifdef WANT_SESSION
+static void	win_session_file_write(void);
+#else
+# define	win_session_file_write()
+#endif
 
 
 /*** global function definitions ***/
@@ -396,6 +406,7 @@ win_foreach(void (*func)(void *))
 	while (wlp) {
 		wp = (win_t *)(wlp->data);
 		func(wp);
+		win_set_title(DOC_CURRENT(wp));
 		wlp = wlp->next;
 	}
 } /* win_foreach */
@@ -602,8 +613,7 @@ win_drag_data_recv(GtkWidget *wgt, GdkDragContext *context, int x, int y,
 	count = g_list_length(fnames);
 	if (count > 0) {
 		msgbar_printf(w, _("Opening files from drag and drop..."));
-		while (gtk_events_pending())
-			gtk_main_iteration_do(TRUE);
+		gtk_events_flush();
 		/*
 		 * we temporarily disable the switch_page signal to avoid
 		 * loading files that are going to be closed anyway.
@@ -750,7 +760,7 @@ gnome_uri_list_extract_filenames (const gchar* uri_list)
 void
 win_close_cb(GtkWidget *wgt, gpointer cbdata)
 {
-	(void)win_close_common((win_t *)cbdata);
+	(void)win_close_common((win_t *)cbdata, FALSE);
 	/* NOT REACHED */
 } /* win_close_cb */
 
@@ -800,8 +810,9 @@ win_close_all_cb(GtkWidget *wgt, gpointer cbdata)
 		}
 	}
 
+	win_session_file_write();
 	while (winlist) {
-		if (win_close_common((win_t *)(winlist->data)) == FALSE)
+		if (win_close_common((win_t *)(winlist->data), TRUE) == FALSE)
 			break;
 	}
 
@@ -984,7 +995,7 @@ win_wwrap_toggle(GtkWidget *wgt, gpointer cbdata)
 static int
 win_delete_event_cb(GtkWidget *wgt, GdkEvent *event, gpointer cbdata)
 {
-	(void)win_close_common((win_t *)cbdata);
+	(void)win_close_common((win_t *)cbdata, FALSE);
 	return TRUE;	/* NOT REACHED */
 } /* win_delete_event_cb */
 
@@ -996,18 +1007,26 @@ win_delete_event_cb(GtkWidget *wgt, GdkEvent *event, gpointer cbdata)
  * and win_close_all_cb().
  */
 static bool_t
-win_close_common(win_t *w)
+win_close_common(win_t *w, bool_t exiting)
 {
 #if defined(USE_TOOLBARS) && defined(USE_HTMLTAGS)
 	GSList *tblp;
 	tblist_data_t *tbldp;
 #endif
 	GSList *dlp, *wlp;
-	int a, b;
 
 	doc_list_destroy(NULL, w);
 	win_list_destroy(NULL, NULL);
 
+#ifdef WANT_SESSION
+	/*
+	 * Exiting is TRUE only from win_close_all_cb, in which case, the
+	 * session has already been saved.
+	 */
+	if (!exiting && (g_slist_length(winlist) < 2))
+		win_session_file_write();
+#endif
+
 	dlp = (GSList *)w->doclist;
 	if (dlp == NULL || w->numdoc < 1)
 		return TRUE;
@@ -1028,15 +1047,10 @@ win_close_common(win_t *w)
 			    prefs.win_xpos, prefs.win_ypos));
 	}
 
-	if (IS_SAVE_WIN_WIDTH() || IS_SAVE_WIN_HEIGHT()) {
-		gdk_window_get_size(w->toplev->window, &a, &b);
-		if (prefs.win_width > 0)
-			prefs.win_width = a;
-		if (prefs.win_height > 0)
-			prefs.win_height = b;
-		GNPDBG_WIN(("win_close_common: width = %d, height = %d\n",
-			    prefs.win_width, prefs.win_height));
-	}
+	gdk_window_get_size(w->toplev->window,
+			    &prefs.win_width, &prefs.win_height);
+	GNPDBG_WIN(("win_close_common: width = %d, height = %d\n",
+		    prefs.win_width, prefs.win_height));
 
 	/* remove from the win list popup window */
 	win_list_remove(w);
@@ -1133,21 +1147,72 @@ notebook_init(win_t *w)
  *
  * sets the window title.
  */
+#define MAX_HOST_LEN	256
 void
 win_set_title(doc_t *d)
 {
-	char *title, *fullpath;
-	int len;
+#ifdef HAVE_SYS_UTSNAME_H
+	struct utsname un;
+#else
+	char hname[MAX_HOST_LEN];
+#endif
+	char *title;
+	char *fullpath = NULL, *user = NULL, *host = NULL;
+	int len = strlen(APP_NAME);
 
-	if (d && (fullpath = file_full_pathname_make(d->fname))) {
-		len = strlen(fullpath) + strlen(APP_NAME) + 5;
-		title = g_new(char, len);
-		g_snprintf(title, len, "%s - %s", APP_NAME, fullpath);
+	if (IS_USER_NAME_IN_TITLE() && (user = getenv("USER")))
+		len += strlen(user);
+
+	if (IS_HOST_NAME_IN_TITLE()) {
+#ifdef HAVE_SYS_UTSNAME_H
+		if (!uname(&un))
+			host = un.nodename;
+#else
+# ifdef HAVE_SYS_SYSTEMINFO_H
+#  define gethostname(nam, len) sysinfo(SI_HOSTNAME, nam, len)
+# endif
+		gethostname(hname, MAX_HOST_LEN);
+		host = hname;
+#endif
+		len += strlen(host) + 1;
+	}
+
+	if ((IS_USER_NAME_IN_TITLE() && user) ||
+	    (IS_HOST_NAME_IN_TITLE() && host))
+		len += 4;
+
+	if (d && (fullpath = file_full_pathname_make(d->fname)))
+		len += strlen(fullpath) + 5;
+	else
+		len += 4;
+
+	title = g_new(char, len);
+	title[0] = '\0';
+
+	if ((IS_USER_NAME_IN_TITLE() && user) ||
+	    (IS_HOST_NAME_IN_TITLE() && host))
+		strcat(title, "(");
+	if (IS_USER_NAME_IN_TITLE() && user)
+		strcat(title, user);
+	if (IS_HOST_NAME_IN_TITLE() && host) {
+		strcat(title, "@");
+		strcat(title, host);
+	}
+
+	if ((IS_USER_NAME_IN_TITLE() && user) ||
+	    (IS_HOST_NAME_IN_TITLE() && host))
+		strcat(title, ") ");
+
+	if (d && fullpath) {
+		strcat(title, APP_NAME);
+		strcat(title, " - ");
+		strcat(title, fullpath);
 		g_free(fullpath);
-		gtk_window_set_title(GTK_WINDOW(d->w->toplev), title);
-		g_free(title);
 	} else
-		gtk_window_set_title(GTK_WINDOW(d->w->toplev), APP_NAME);
+		strcat(title, APP_NAME);
+
+	gtk_window_set_title(GTK_WINDOW(d->w->toplev), title);
+	g_free(title);
 } /* win_set_title */
 
 
@@ -1207,16 +1272,40 @@ win_list_set_numdoc(win_t *w)
 void
 win_list_show(GtkWidget *wgt, gpointer cbdata)
 {
+#ifdef USE_LIBGLADE
+	GladeXML *xml;
+	GtkWidget *tmp;
+#else
+	char *titles[]= { N_(" # "), N_(" # Docs "), N_(" Current Document ") };
+	int i;
+#endif
 	GSList *wlp;
 	win_t *wp;
-	int i;
-
-	char *titles[]= { N_(" # "), N_(" # Docs "), N_(" Current Document ") };
 
 	if (misc_show_and_raise(wlw))
 		return;
-	g_assert(w != NULL);
 
+#ifdef USE_LIBGLADE
+	xml = my_glade_xml_get("Win List");
+	wlw = my_glade_widget_get(xml, "Win List", GLADE_POS_MOUSE);
+	wlw_data = my_glade_widget_get(xml, "winlistclist", 0);
+	(void)gtk_signal_connect(GTK_OBJECT(wlw), "destroy",
+				 GTK_SIGNAL_FUNC(win_list_destroy), NULL);
+	(void)gtk_signal_connect(GTK_OBJECT(wlw_data), "select_row",
+				 GTK_SIGNAL_FUNC(win_list_select), NULL);
+	tmp = my_glade_widget_get(xml, "closeselected", 0);
+	(void)gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
+				 GTK_SIGNAL_FUNC(win_list_close_selected),NULL);
+	tmp = my_glade_widget_get(xml, "closeall", 0);
+	(void)gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
+				 GTK_SIGNAL_FUNC(win_close_all_cb), NULL);
+	tmp = my_glade_widget_get(xml, "exitwinlist", 0);
+	(void)gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
+				 GTK_SIGNAL_FUNC(win_list_destroy), NULL);
+	gtk_window_set_default_size(GTK_WINDOW(wlw),
+				    prefs.winlist_w, prefs.winlist_h);
+	gtk_object_destroy(GTK_OBJECT(xml));
+#else
 	for (i = 0; i < sizeof(titles)/sizeof(char*); i++)
 		titles[i] = gettext(titles[i]);
 
@@ -1227,6 +1316,7 @@ win_list_show(GtkWidget *wgt, gpointer cbdata)
 			win_list_select, NULL,
 			win_list_close_selected, NULL,
 			win_close_all_cb, NULL);
+#endif
 
 	/* for each window, add info into the clist */
 	gtk_clist_freeze(GTK_CLIST(wlw_data));
@@ -1282,7 +1372,7 @@ win_list_close_selected(GtkWidget *wgt, gpointer cbdata)
 	while (wlp) {
 		w = (win_t *)(wlp->data);
 		if (w->rownum == curwinrow) {
-			if (win_close_common(w) == TRUE)
+			if (win_close_common(w, FALSE) == TRUE)
 				break;
 		}
 		wlp = wlp->next;
@@ -1438,4 +1528,166 @@ win_key_press(GtkWidget *txt, GdkEventKey *event, win_t *w)
 #endif
 
 
+#ifdef WANT_SESSION
+static void
+win_session_file_write(void)
+{
+	FILE *fp;
+	win_t *w;
+	doc_t *d;
+	char *fpath;
+	GSList *wp, *dp;
+	int xpos, ypos, width, height;
+
+	fp = file_open_fp(prefs.sessionrc, "w", OPEN_EXCL,
+			  "win_session_file_write");
+	if (fp == NULL)
+		return;
+
+	for (wp = winlist; wp; wp = wp->next) {
+		w = (win_t *)wp->data;
+		gdk_window_get_position(w->toplev->window, &xpos, &ypos);
+		gdk_window_get_size(w->toplev->window, &width, &height);
+		fprintf(fp, "\t\t%d %d %d %d %d\n",
+			w->numdoc, xpos, ypos, width, height);
+
+		for (dp = w->doclist; dp; dp = dp->next) {
+			d = (doc_t *)dp->data;
+
+			gtk_events_flush();
+			if (!d->fname) {
+				fprintf(fp, "\n");
+				continue;
+			}
+
+			if (my_path_is_absolute(d->fname))
+				fprintf(fp, "%s\n", d->fname);
+			else {
+				fpath = file_full_pathname_make(d->fname);
+				fprintf(fp, "%s\n", fpath);
+				g_free(fpath);
+			}
+		}
+	}
+
+	file_close_fp(prefs.sessionrc, fp);
+} /* win_session_file_write */
+
+
+#ifdef USE_GNOME
+/*
+ * Callback for "save_yourself" signal, for GNOME session management.
+ */
+int
+win_session_save(GnomeClient *client, int phase, GnomeSaveStyle save_style,
+		 int is_shutdown, GnomeInteractStyle interact_style,
+		 int is_fast, gpointer client_data)
+{
+	char **argv;
+	guint argc;
+
+	argc = 2;
+	argv = g_malloc(sizeof(char *) * 3);
+	argv[0] = client_data;
+	argv[1] = "--ldsession";
+	argv[2] = NULL;
+
+	gnome_client_set_clone_command(client, argc, argv);
+	gnome_client_set_restart_command(client, argc, argv);
+
+	g_free(argv);
+
+	win_session_file_write();
+
+	return TRUE;
+} /* win_session_save */
+
+
+void
+win_session_die(GnomeClient *client, gpointer client_data)
+{
+	gtk_main_quit();
+}
+#endif	/* USE_GNOME */
+
+
+static void
+win_session_load_complete(void *wp)
+{
+	win_t *w = (win_t *)wp;
+
+	doc_load(DOC_CURRENT(w));
+	win_set_title(DOC_CURRENT(w));
+
+	/* see app_init() as to why we connect this signal here */
+	(void)gtk_signal_connect_after(GTK_OBJECT(w->nb), "switch_page",
+				       GTK_SIGNAL_FUNC(doc_switch_page), w);
+	gtk_widget_show(w->toplev);
+} /* win_session_load_complete */
+
+
+/*
+ * returns TRUE upon successful loading, FALSE if not.
+ */
+bool_t
+win_session_load(void)
+{
+	char buf[MAXPATH + 1];
+	int added, num, xpos, ypos, width, height;
+	FILE *fp;
+	win_t *w;
+
+	if (!prefs.sessionrc || !file_exist(prefs.sessionrc))
+		return FALSE;
+
+	fp = file_open_fp(prefs.sessionrc, "r", 0, "win_session_load");
+	if (fp == NULL)
+		return FALSE;
+
+	w = NULL;
+	num = 0;
+	added = 0;
+	while (fgets(buf, MAXPATH + 1, fp)) {
+		if (buf[0] == '\t' && buf[1] == '\t') {
+			w = win_new();
+			sscanf(buf, "%d %d %d %d %d",
+			       &num, &xpos, &ypos, &width, &height);
+			gtk_widget_set_uposition(w->toplev, xpos, ypos);
+#ifdef GTK_HAVE_FEATURES_1_1_0
+			gtk_window_set_default_size(GTK_WINDOW(w->toplev),
+						    width, height);
+#else
+			gtk_widget_set_usize(GTK_WIDGET(w->toplev),
+					     width, height);
+#endif
+		} else if (buf[0] != '\n') {
+			g_assert(w);
+			g_assert(num >= 0);
+			num--;
+			if (buf[strlen(buf) - 1] == '\n')
+				buf[strlen(buf) - 1] = '\0';
+
+			if (!file_exist(buf))
+				continue;
+
+			added++;
+			doc_new(w, buf, DocText, num == 0 ? DO_LOAD : 0);
+		} else if (buf[0] == '\n') {	/* empty line (no document) */
+			g_assert(w);
+			g_assert(num >= 0);
+			num--;
+			added++;
+			doc_new(w, NULL, DocText, 0);
+		}
+	}
+
+	file_close_fp(prefs.sessionrc, fp);
+
+	win_foreach(win_session_load_complete);
+
+	return TRUE;
+} /* win_session_load */
+#endif	/* WANT_SESSION */
+
+
 /* the end */
